@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 import os, time, re
 from functools import partial
-from flask import Flask, Module, url_for, render_template, request, session, redirect, g, make_response, current_app
+from flask import Flask, Module, url_for, request, session, redirect, g, make_response, current_app, render_template
 from decorators import login_required, guest_or_login_required, with_lock
 from decorators import global_lock
 # Make flask use the old session foo from <=flask-0.9
@@ -15,8 +15,11 @@ except ImportError:
 SRC = os.path.join(SAGE_SRC, 'sage')
 from flask.ext.openid import OpenID
 from flask.ext.babel import Babel, gettext, ngettext, lazy_gettext, get_locale
-from sagenb.misc.misc import SAGENB_ROOT, DATA, SAGE_DOC, translations_path, N_, nN_
-
+from sagenb.misc.misc import SAGENB_ROOT, DATA, SAGE_DOC, translations_path, N_, nN_, unicode_str
+from json import dumps
+from sagenb.notebook.cell import number_of_rows
+from sagenb.notebook.template import (css_escape, clean_name,
+                                      prettify_time_ago, TEMPLATE_PATH)
 oid = OpenID()
 
 class SageNBFlask(Flask):
@@ -55,10 +58,17 @@ class SageNBFlask(Flask):
         self.add_static_path('/doc/static', DOC)
         #self.add_static_path('/doc/static/reference', os.path.join(SAGE_DOC, 'reference'))
 
-    def create_jinja_environment(self):
-        from sagenb.notebook.template import env
-        env.globals.update(url_for=url_for)
-        return env
+        # Template globals
+        self.add_template_global(url_for)
+        # Template filters
+        self.add_template_filter(css_escape)
+        self.add_template_filter(number_of_rows)
+        self.add_template_filter(clean_name)
+        self.add_template_filter(prettify_time_ago)
+        self.add_template_filter(max)
+        self.add_template_filter(lambda x: repr(unicode_str(x))[1:],
+                                 name='repr_str')
+        self.add_template_filter(dumps, 'tojson')
 
     def static_view_func(self, root_path, filename):
         from flask.helpers import send_from_directory
@@ -70,8 +80,9 @@ class SageNBFlask(Flask):
                           view_func=partial(self.static_view_func, root_path))
 
     def message(self, msg, cont='/', username=None, **kwds):
+        from sagenb.misc.misc import SAGE_VERSION
         """Returns an error message to the user."""
-        template_dict = {'msg': msg, 'cont': cont, 'username': username}
+        template_dict = {'msg': msg, 'cont': cont, 'username': username, 'sage_version': SAGE_VERSION}
         template_dict.update(kwds)
         return render_template(os.path.join('html', 'error_message.html'),
                                **template_dict)
@@ -334,7 +345,6 @@ def set_profiles():
             return render_template('html/accounts/openid_profile.html', **parse_dict)
         return redirect(url_for('base.index'))
 
-
 #############
 # OLD STUFF #
 #############
@@ -410,7 +420,8 @@ def create_app(path_to_notebook, *args, **kwds):
     ##############
     # Create app #
     ##############
-    app = SageNBFlask('flask_version', startup_token=startup_token)
+    app = SageNBFlask('flask_version', startup_token=startup_token,
+                      template_folder=TEMPLATE_PATH)
     app.secret_key = os.urandom(24)
     oid.init_app(app)
     app.debug = True
